from snippet_management.code_snippet import CodeSnippet
from dataclasses import dataclass, field
from typing import Dict, Union


@dataclass
class SnippetStorage:
    _storage: Dict[int, CodeSnippet] = field(default_factory=dict)

    def __contains__(self, code_snippet: CodeSnippet):
        return isinstance(self.get_code_snippet(code_snippet), CodeSnippet)

    def __len__(self):
        return len(self.storage)

    @property
    def storage(self) -> Dict[int, CodeSnippet]:
        return self._storage

    def update_storage(self, snippets_dict: Dict[int, CodeSnippet]):
        self.storage.update(snippets_dict)

    def get_code_snippet(self, code_snippet: CodeSnippet) -> Union[CodeSnippet, None]:
        hashed_snippet = hash(code_snippet)
        if hashed_snippet in self._storage:
            return self._storage.get(hashed_snippet)
        else:
            return None

    def add_code_snippet(self, code_snippet: CodeSnippet) -> bool:
        if code_snippet in self:
            return False
        hashed_snippet = hash(code_snippet)
        self._storage.update({hashed_snippet: code_snippet})
        return True

    def update_code_snippet(self, code_snippet: CodeSnippet) -> bool:
        # only changes the implementation
        if code_snippet not in self:
            return False
        hashed_snippet = hash(code_snippet)
        self._storage.update({hashed_snippet: code_snippet})
        return True

    def delete_code_snippet(self, code_snippet: CodeSnippet) -> bool:
        if code_snippet not in self:
            return False
        hashed_snippet = hash(code_snippet)
        self._storage.pop(hashed_snippet)
        return True

    def show_storage(self):
        for key, value in (self._storage).items():
            print(f"{key}: {value}")



from git_tools.git_retrieaver import GitRetrieaver
from pathlib import Path


class FileHandler:
    def __init__(self, file_path: Path):
        if file_path.is_file():
            self.file_path = file_path
            self.file_name = self.file_path.name
            self.file_extension = self.file_path.suffix[1:]
            self.file_str = GitRetrieaver.retrieve_file(self.file_path)
        else:
            raise Exception("FileHandler_Path_Error: The given path is not a file")

    def __str__(self):
        string = f"\nfile_path: {self.file_path}\nfile_name: {self.file_name}\nfile_extension: {self.file_extension}\nfile_str:\n\n{self.file_str}\n\n"
        return string



from git_tools.git_manager import GitManager
from doc_ignore.doc_ignore import DocIgnore
from pathlib import Path
from pygit2 import Tree, GIT_OBJ_COMMIT
from typing import List


# Don't work with repos inside repos, polyrepos.
class GitFilePaths:
    doc_ignore = DocIgnore()

    @staticmethod
    def _valid_file(sys_path: Path) -> bool:
        return not (
            (sys_path.name in GitFilePaths.doc_ignore)
            or (sys_path.suffix in GitFilePaths.doc_ignore)
            or (sys_path.name == ".docignore")
            or (sys_path.name == ".git")
            or (sys_path.name == ".gitignore")
            or (sys_path.name == "doc.log")
        )

    @staticmethod
    def get_all_valid_paths(
        git_tree: Tree = None,
        parent_path: Path = Path("."),
    ) -> List[Path]:
        paths: List[Path] = []
        if not git_tree:
            GitManager.select_front_commit()
            git_tree = GitManager.selected_commit_tree()
        for entry in git_tree:
            full_path: Path = parent_path / entry.name
            conditions = (
                not full_path.exists()
                or entry.type == GIT_OBJ_COMMIT
                or not GitFilePaths._valid_file(full_path)
            )
            if conditions:
                continue
            if full_path.is_file():
                paths.append(full_path)
            elif full_path.is_dir():
                dir_paths: List[Path] = GitFilePaths.get_all_valid_paths(
                    GitManager.project_repo()[entry.oid], full_path
                )
                paths.extend(dir_paths)
        return paths



from tree_sitter_logic.tree_sitter_util import get_identifier, get_implementation
from snippet_management.node_info import NodeInfo
from dataclasses import dataclass


@dataclass()
class SnippetExtract:
    parent_type: str
    parent_identifier: str
    snippet_type: str
    snippet_identifier: str
    snippet_implementation: str

    def __init__(self, node_info: NodeInfo, file_str: str):
        self.parent_type = node_info.parent_type
        self.parent_identifier = node_info.parent_identifier
        self.snippet_type = node_info.node.type
        self.snippet_identifier = get_identifier(node_info.node, file_str)
        self.snippet_implementation = get_implementation(node_info.node, file_str)



from snippet_management.snippet_storage import SnippetStorage
from snippet_management.code_snippet import CodeSnippet
from snippet_management.snippet_extract import SnippetExtract
from tree_sitter_logic.languages import FileExtension, LanguageNodes
from tree_sitter_logic.tree_sitter_util import (
    get_specified_nodes,
    global_node_types,
    get_nodes,
)
from file_handler.file_handler import FileHandler
from typing import Set, Dict
from dataclasses import dataclass, field
from tree_sitter import Parser, Node
from snippet_management.dependencies_info import DependenciesInfo


@dataclass
class CodeScrapper:
    _input_file: FileHandler = None
    _file_dependencies: DependenciesInfo = None
    _snippet_storage: SnippetStorage = field(default_factory=lambda: SnippetStorage())
    _parser: Parser = field(default_factory=lambda: Parser())
    _relevant_nodes_names: Set[str] = None
    _assignment_nodes_names: Set[str] = None
    _ast_root: Node = None

    @property
    def storage_dict(self) -> Dict[int, CodeSnippet]:
        return self._snippet_storage.storage

    def extract_snippets(self) -> bool:
        if not self._check_input_file():
            raise Exception("CodeScrapper_File_Error: No file has been charged ")
        self._scrape_relevant()
        self._scrape_assignment()
        return True

    def change_file(self, file: FileHandler) -> bool:
        self._input_file = file
        self._update_scrapper()
        return True

    def _update_scrapper(self):
        self._update_parser()
        self._update_nodes_names()
        self._update_ast_root()
        self._update_file_dependencies()

    def _update_parser(self):
        self._parser.set_language(FileExtension[self._input_file.file_extension].value)

    def _update_nodes_names(self):
        self._relevant_nodes_names = LanguageNodes[
            self._input_file.file_extension
        ].value[0]
        self._assignment_nodes_names = LanguageNodes[
            self._input_file.file_extension
        ].value[1]

    def _update_ast_root(self):
        tree = self._parser.parse(bytes(self._input_file.file_str, "utf8"))
        self._ast_root = tree.root_node

    def _update_file_dependencies(self):
        self._file_dependencies = DependenciesInfo(
            self._ast_root, self._input_file.file_str
        )

    def _scrape_relevant(self):      
        relevant_nodes= get_nodes(
            self._ast_root, self._relevant_nodes_names, self._input_file.file_str
        )
        for node_info in relevant_nodes:
            snippet_extract = SnippetExtract(node_info, self._input_file.file_str)
            code_snippet = CodeSnippet(
                self._input_file, snippet_extract, self._file_dependencies
            )
            self._save_code_snippet(code_snippet) #hola

    def _scrape_assignment(self):
        assignment_nodes = get_nodes(
            self._ast_root, self._assignment_nodes_names, self._input_file.file_str
        )
        for node_info in assignment_nodes:
            snippet_extract = SnippetExtract(node_info, self._input_file.file_str)
            code_snippet = CodeSnippet(
                self._input_file, snippet_extract, self._file_dependencies
            )
            self._save_code_snippet(code_snippet)

    def _save_code_snippet(self, code_snippet: CodeSnippet):
        self._snippet_storage.add_code_snippet(code_snippet)

    def _check_input_file(self):
        return self._input_file != None and isinstance(self._input_file, FileHandler)

    def show_storage(self):
        self._snippet_storage.show_storage()



from repository_scrapping.file_scrapper import FileScrapper
from snippet_management.snippet_storage import SnippetStorage
from git_tools.git_file_checker import GitFileChecker
from git_tools.git_manager import GitManager
from metaclasses.singleton_meta import SingletonMeta
from typing import ClassVar


class AddedFilesManager(metaclass=SingletonMeta):
    instance: ClassVar
    _added_file_scrapper: FileScrapper
    _snippets_to_doc: SnippetStorage

    def __init__(self):
        self._added_file_scrapper = FileScrapper()
        self._snippets_to_doc = SnippetStorage()

    def _start_added_file_scrapper(self):
        if GitFileChecker.added:
            GitManager.select_front_commit()
            self._added_file_scrapper.scrape_specified(GitFileChecker.added)

    def _update_snippets_to_doc(self):
        self._start_added_file_scrapper()
        snippets_to_doc_dict = self._added_file_scrapper.storage_dict
        self._snippets_to_doc.update_storage(snippets_to_doc_dict)

    @staticmethod
    def get_snippets_to_doc() -> SnippetStorage:
        return AddedFilesManager.instance._snippets_to_doc

    @staticmethod
    def check_added_files():
        AddedFilesManager.instance._update_snippets_to_doc()


AddedFilesManager = AddedFilesManager()



from git_tools.git_manager import GitManager
from metaclasses.no_instanciable_meta import NoInstanciable
from pygit2 import Diff, Commit, GIT_DELTA_ADDED, GIT_DELTA_MODIFIED, GIT_DELTA_DELETED
from pathlib import Path
from typing import List


class GitFileChecker(metaclass=NoInstanciable):
    added: List[Path] = []
    modified: List[Path] = []
    deleted: List[Path] = []
    files_diff: Diff

    @staticmethod
    def update_changed_files():
        tail_commit = GitManager.tail_commit()
        front_commit = GitManager.front_commit()
        GitFileChecker.files_diff = GitFileChecker._get_diff(tail_commit, front_commit)
        GitFileChecker._extract_changes()

    @staticmethod
    def _get_diff(tail_commit: Commit, head_commit: Commit) -> Diff:
        return GitManager.project_repo().diff(tail_commit, head_commit)

    @staticmethod
    def _extract_changes():
        for patch in GitFileChecker.files_diff:
            if patch.delta.status == GIT_DELTA_ADDED:
                GitFileChecker.added.append(Path(patch.delta.new_file.path))
            elif patch.delta.status == GIT_DELTA_MODIFIED:
                GitFileChecker.modified.append(Path(patch.delta.new_file.path))
            elif patch.delta.status == GIT_DELTA_DELETED:
                GitFileChecker.deleted.append(Path(patch.delta.old_file.path))



from tree_sitter_logic.tree_sitter_util import get_dependencies, get_implementation
from dataclasses import dataclass
from tree_sitter import Node


@dataclass()
class DependenciesInfo:
    dependencies: str

    def __init__(self, node: Node, file_str: str):
        self.dependencies = self._get_imports_string(node, file_str)

    def _get_imports_string(self, node: Node ,file_str: str) -> str:
        dependencies_nodes = get_dependencies(node)
        import_statements = []
        for dependency in dependencies_nodes:
            import_statements.append(get_implementation(dependency, file_str))
        imports_string = '\n'.join(import_statements)
        return imports_string



from repository_scrapping.code_scrapper import CodeScrapper
from snippet_management.code_snippet import CodeSnippet
from file_handler.file_handler import FileHandler
from doc_ignore.doc_ignore import DocIgnore
from git_tools.git_manager import GitManager
from pathlib import Path
from typing import List, Dict


class FileScrapper:
    _root_dir: Path
    _current_file: FileHandler
    _code_scrapper: CodeScrapper
    _ignore: DocIgnore

    def __init__(self, root_dir: Path = Path(".")):
        self._root_dir = root_dir
        self._ignore = DocIgnore(root_dir)
        self._code_scrapper = CodeScrapper()
        self._current_file = None

    def scrape_specified(self, specified_files: List[Path]) -> bool:
        for file_path in specified_files:
            if self._valid_file(file_path):
                self._start_scrape(file_path)
        return True

    @property
    def storage_dict(self) -> Dict[int, CodeSnippet]:
        return self._code_scrapper.storage_dict

    def _update_current_file(self, file_path: Path):
        self._current_file = FileHandler(file_path)

    def _start_scrape(self, file_path: Path):
        self._update_current_file(file_path)
        self._code_scrapper.change_file(self._current_file)
        self._code_scrapper.extract_snippets()

    def _valid_file(self, sys_path: Path) -> bool:
        divide_path = str(sys_path).split("\\")
        return not (
            (any(path_name in self._ignore for path_name in divide_path))
            or (sys_path.suffix in self._ignore)
            or (sys_path.name == ".docignore")
            or (sys_path.name == ".git")
            or (sys_path.name == ".gitignore")
            or (sys_path.name == "doc.log")
        )



from git_tools.git_manager import GitManager
from metaclasses.no_instanciable_meta import NoInstanciable
from pathlib import Path
from pygit2 import Blob


class GitRetrieaver(metaclass=NoInstanciable):
    @staticmethod
    def retrieve_file(file_path: Path) -> str:
        file_git_id = (GitRetrieaver.get_file_git_object(file_path)).oid
        file_git_blob = GitManager.project_repo()[file_git_id]
        file_bytes_data = file_git_blob.data
        file_worked_data = file_bytes_data.decode("utf-8")
        return file_worked_data

    @staticmethod
    def get_file_git_object(file_path: Path) -> Blob:
        path_way = str(file_path).split("\\")
        current_object = GitManager.selected_commit_tree()
        for path in path_way:
            if path in current_object:
                current_object = current_object[path]
            else:
                raise Exception("Git_Retriever_Error: Not object found in git tree.")
        return current_object



from snippet_management.snippet_extract import SnippetExtract
from snippet_management.dependencies_info import DependenciesInfo
from file_handler.file_handler import FileHandler
from dataclasses import dataclass, field, asdict
import json
from pathlib import Path

@dataclass
class CodeSnippet:
    file_path: Path
    file_name: str
    file_extension: str
    dependencies: str
    parent_type: str
    parent_identifier: str
    code_snippet_type: str
    code_snippet_identifier: str
    code_snippet_implementation: str = field(hash=False)

    def __init__(
        self,
        file_handler: FileHandler,
        snippet_extract: SnippetExtract,
        dependencies_info: DependenciesInfo,
    ):
        self.file_path = file_handler.file_path
        self.file_name = file_handler.file_name
        self.file_extension = file_handler.file_extension
        self.dependencies = dependencies_info.dependencies
        self.parent_type = snippet_extract.parent_type
        self.parent_identifier = snippet_extract.parent_identifier
        self.code_snippet_type = snippet_extract.snippet_type
        self.code_snippet_identifier = snippet_extract.snippet_identifier
        self.code_snippet_implementation = snippet_extract.snippet_implementation

    def __hash__(self):
        return hash(
            (
                self.file_path,
                self.file_name,
                self.file_extension,
                self.code_snippet_type,
                self.code_snippet_identifier,
            )
        )

    def __str__(self):
        string = f"\n\nfile_path: {self.file_path}\nfile_name: {self.file_name}\nfile_extension: {self.file_extension}\n\ndependencies:\n{self.dependencies}\n\nparent_type: {self.parent_type}\nparent_identifier: {self.parent_identifier}\nsnippet_type: {self.code_snippet_type}\nsnippet_identifier: {self.code_snippet_identifier}\nsnippet_implementation:\n\n{self.code_snippet_implementation}\n"
        return string

    @property
    def implementation(self):
        return self.code_snippet_implementation

    def json(self) -> str:
        return json.dumps(asdict(self))

    @staticmethod
    def to_code_snippet(snippet_json: str):
        return CodeSnippet(**(json.loads(snippet_json)))



from dataclasses import dataclass
from tree_sitter import Node


@dataclass()
class NodeInfo:
    node: Node
    parent_type: str
    parent_identifier: str

    def __init__(
        self, child_node: Node, parent_node: "NodeInfo" = None, file_str: str = None
    ):
        self.file_str = file_str
        self.node = child_node
        if parent_node is None:
            self.parent_type = "program"
            self.parent_identifier = "program"
        elif parent_node.parent_type == "program":
            self.parent_type = "root node"
            self.parent_identifier = "root node"
        else:
            parent_identifier = NodeInfo._get_identifier(parent_node.node, file_str)
            if parent_identifier is None:
                self.parent_type = parent_node.parent_type
                self.parent_identifier = parent_node.parent_identifier
            else: 
                self.parent_type = parent_node.node.type
                self.parent_identifier = parent_identifier

    def __str__(self):
        return f"\nnode_identifier:{NodeInfo._get_identifier(self.node, self.file_str)}\nparent_identifier:{self.parent_identifier}\parent_type:{self.parent_type}\nnode_type={self.node.type}"

    def __repr__(self):
        return self.__str__()

    @property
    def children(self):
        return self.node.children

    @staticmethod
    def _get_identifier(node: Node, file_str: str) -> str:
        for child in node.children:
            if child.type == "identifier":
                return file_str[child.start_byte : child.end_byte]
        return None



from ai_logic.documentor_chat import DocumentorChat
from snippet_management.code_snippet import CodeSnippet
from documentation_orchestrator.documentation_manager import DocumentationManager
from typing import Dict, ClassVar
from pathlib import Path
from metaclasses.singleton_meta import SingletonMeta


class DocWriter(metaclass=SingletonMeta):
    instance: ClassVar
    doc_path: Path = Path("./docs")
    snippets_to_doc: Dict[int, CodeSnippet] = None
    current_snippet: CodeSnippet = None
    current_doc_path: Path = None
    snippet_documentation: str = None

    def __init__(self):
        self.doc_path = Path("./docs")
        self.snippets_to_doc = None
        self.current_snippet = None
        self.current_doc_path = None
        self.snippet_documentation = None

    @staticmethod
    def start_writing_documentation():
        DocWriter.instance._main()

    def _main(self):
        self._get_snippets_to_doc()
        for snippet_to_doc in self.snippets_to_doc.values():
            self._set_current_snippet(snippet_to_doc)
            self._document_current_snippet()

    def _get_snippets_to_doc(self):
        self.snippets_to_doc = DocumentationManager.get_snippets_to_doc().storage

    def _set_current_snippet(self, snippet_to_doc: CodeSnippet):
        self.current_snippet = snippet_to_doc

    def _document_current_snippet(self):
        self._get_current_snippet_documentation()
        self._set_current_doc_path()
        self._create_doc_md()

    def _get_current_snippet_documentation(self):
        DocumentorChat.set_snippet_to_doc(self.current_snippet)
        DocumentorChat.ask_documentation()
        self.snippet_documentation = DocumentorChat.get_documentation_answer()

    def _set_current_doc_path(self):
        file_path = self.current_snippet.file_path.parent
        file_name_without_extension = str(self.current_snippet.file_path.stem)
        file_doc = Path(file_name_without_extension + ".md")
        doc_file_path = self.doc_path / file_path / file_doc
        self.current_doc_path = doc_file_path

    def _set_current_doc_path(self):
        file_path = self.current_snippet.file_path.parent
        file_name = str(self.current_snippet.file_path.stem)
        file_doc = Path(self.current_snippet.code_snippet_identifier + ".md")
        doc_file_path = self.doc_path / file_path / file_name 
        if self.current_snippet.parent_type == "class_definition":
            doc_file_path = doc_file_path / f"{self.current_snippet.parent_identifier}"
        doc_file_path = doc_file_path / file_doc
        self.current_doc_path = doc_file_path

    def _create_doc_md(self):
        self.current_doc_path.parent.mkdir(parents=True, exist_ok=True)
        self.current_doc_path.touch(exist_ok=True)
        self.current_doc_path.write_text(self.snippet_documentation, encoding="utf-8")


DocWriter()



from snippet_management.snippet_storage import SnippetStorage
from git_tools.git_manager import GitManager
from git_tools.git_retrieaver import GitRetrieaver
from textwrap import dedent
from pathlib import Path

# snippets to delete does not count the deleted files


class DocLog:
    @staticmethod
    def create_doc_log():
        desired_path = DocLog.get_doc_log_path()
        desired_path.touch()

    @staticmethod
    def update_doc_log(
        snippets_to_doc: SnippetStorage, snippets_to_delete: SnippetStorage
    ):
        DocLog.create_doc_log()
        num_snippets_to_doc = len(snippets_to_doc)
        num_snippets_to_delete = len(snippets_to_delete)
        head_commit_hash: str = GitManager.head_commit().short_id
        new_log_message = dedent(
            f"""
        COMMIT_HASH:{head_commit_hash}
        NUMBER_OF_SNIPPETS_TO_DOC:{num_snippets_to_doc}
        NUMBER_OF_SNIPPETS_TO DELETE:{num_snippets_to_delete}
        """
        )
        doc_log_path = DocLog.get_doc_log_path()
        original_log_content = doc_log_path.read_text()
        doc_log_path.write_text(new_log_message + original_log_content)

    @staticmethod
    def exists_doc_log() -> bool:
        desired_path = Path("./doc.log")
        try:
            GitManager.select_front_commit()
            GitRetrieaver.get_file_git_object(desired_path)
            return True
        except Exception:
            return False

    @staticmethod
    def get_doc_log_path() -> Path:
        return Path("./doc.log")



from metaclasses.singleton_meta import SingletonMeta
from git_tools.git_retrieaver import GitRetrieaver
from git_tools.git_manager import GitManager
from pathlib import Path
from typing import ClassVar


# .docignore file must be found at the root directory
class DocIgnore(metaclass=SingletonMeta):
    instance: ClassVar

    def __init__(self, root_path: Path = None):
        project_root = root_path if root_path != None else Path(".")
        doc_ignore_path = project_root / ".docignore"
        if doc_ignore_path.exists():
            GitManager.select_front_commit()
            read_doc = GitRetrieaver.retrieve_file(doc_ignore_path)
            doc_lines = read_doc.splitlines()
            doc_set = set(doc_lines)
            clean_doc_set = {line.strip() for line in doc_set}
            self.ignore = clean_doc_set
            GitManager.select_tail_commit()
        else:
            raise Exception(
                "DocIgnore_.docignore_not_found: .docignore is not at the root dir"
            )

    def __contains__(self, sys_object: str):
        return sys_object in self.ignore



from documentation_orchestrator.modified_files_manager import ModifiedFilesManager
from documentation_orchestrator.added_files_manager import AddedFilesManager
from documentation_orchestrator.first_run_manager import FirstRunManager
from snippet_management.snippet_storage import SnippetStorage
from git_tools.git_file_checker import GitFileChecker
from git_tools.git_manager import GitManager
from metaclasses.singleton_meta import SingletonMeta
from doc_log.doc_log import DocLog
from typing import List, ClassVar
from pathlib import Path


# GitFileChecker. Solo se ha de usar su update aqui
class DocumentationManager(metaclass=SingletonMeta):
    instance: ClassVar
    _snippets_to_doc: SnippetStorage
    _snippets_to_delete: SnippetStorage
    _files_to_delete: List[Path]

    def __init__(self):
        self._snippets_to_doc = SnippetStorage()
        self._snippets_to_delete = SnippetStorage()
        self._files_to_delete = None

    def _update_snippets_to_doc(self):
        added_file_snippets_dict = AddedFilesManager.get_snippets_to_doc().storage
        self._snippets_to_doc.update_storage(added_file_snippets_dict)
        modified_file_snippets_dict = ModifiedFilesManager.get_snippets_to_doc().storage
        self._snippets_to_doc.update_storage(modified_file_snippets_dict)

    def _update_snippets_to_doc_first_run(self):
        all_snippets_to_doc = FirstRunManager.get_snippets_to_doc().storage
        self._snippets_to_doc.update_storage(all_snippets_to_doc)

    def _update_snippets_to_delete(self):
        deleted_file_snippets_dict = (
            ModifiedFilesManager.get_snippets_to_delete().storage
        )
        self._snippets_to_delete.update_storage(deleted_file_snippets_dict)

    def _update_files_to_delete(self):
        self._files_to_delete = GitFileChecker.deleted

    @staticmethod
    def start_documentation():
        if not DocLog.exists_doc_log():
            DocumentationManager._run_first_diagnosis()
        else:
            DocumentationManager._run_diagnosis()

    @staticmethod
    def update_doc_log():
        to_doc = DocumentationManager.instance._snippets_to_doc
        to_delete = DocumentationManager.instance._snippets_to_delete
        DocLog.update_doc_log(to_doc, to_delete)

    @staticmethod
    def _run_first_diagnosis():
        FirstRunManager.start_first_run()
        DocumentationManager.instance._update_snippets_to_doc_first_run()
        DocumentationManager.update_doc_log()

    @staticmethod
    def _run_diagnosis():
        GitFileChecker.update_changed_files()
        AddedFilesManager.check_added_files()
        ModifiedFilesManager.check_modified_files()
        DocumentationManager.instance._update_snippets_to_doc()
        DocumentationManager.instance._update_snippets_to_delete()
        DocumentationManager.instance._update_files_to_delete()
        DocumentationManager.update_doc_log() # cambio

    @staticmethod
    def get_snippets_to_doc() -> SnippetStorage:
        return DocumentationManager.instance._snippets_to_doc

    @staticmethod
    def get_snippets_to_delete() -> SnippetStorage:
        return DocumentationManager.instance._snippets_to_delete

    @staticmethod
    def show_results():
        print("-------------------------------------------------------------\n")
        print("\t\tSNIPPETS TO DOCUMENTATE\n")
        print("-------------------------------------------------------------\n")
        DocumentationManager.get_snippets_to_doc().show_storage()
        print("\n\n\n\n")
        print("-------------------------------------------------------------\n")
        print("\t\tSNIPPETS TO DELETE\n")
        print("-------------------------------------------------------------\n")
        DocumentationManager.get_snippets_to_delete().show_storage()
        print("-------------------------------------------------------------\n")
        print("\n\n\nEND")


DocumentationManager()



import os
from metaclasses.singleton_meta import SingletonMeta
from langchain.chat_models import ChatOpenAI
from langchain.schema import HumanMessage
from langchain.prompts import (
    SystemMessagePromptTemplate,
    HumanMessagePromptTemplate,
    ChatPromptTemplate,
    PromptTemplate,
)
from snippet_management.code_snippet import CodeSnippet
from ai_logic.doc_example import (
    CLASS_DOCUMENTATION_TEMPLATE,
    METHOD_DOCUMENTATION_TEMPLATE,
    FUNCTION_DOCUMENTATION_TEMPLATE,
)
from typing import ClassVar

class DocumentorChat(metaclass=SingletonMeta):
    instance: ClassVar = None
    chat: ChatOpenAI
    system_message: SystemMessagePromptTemplate
    snippet_to_doc: CodeSnippet
    current_doc_message: HumanMessage
    current_chat_message: ChatPromptTemplate
    current_answer: str = None

    def __init__(self) -> None:
        key = os.environ.get('OPENAI_API_KEY')
        self.chat = ChatOpenAI(model_name="gpt-3.5-turbo", openai_api_key=key)

    @staticmethod
    def set_snippet_to_doc(snippet_to_doc: CodeSnippet):
        DocumentorChat.instance._set_snippet_to_doc(snippet_to_doc)

    @staticmethod
    def ask_documentation():
        DocumentorChat.instance._ask_documentation()
    
    @staticmethod
    def get_documentation_answer() -> str:
        return DocumentorChat.instance.current_answer

    def _set_snippet_to_doc(self, snippet_to_doc: CodeSnippet):
        self.snippet_to_doc = snippet_to_doc
        self._set_correct_templates()

    def _ask_documentation(self):
        self.current_answer = self.chat(self.current_chat_message).content

    def _set_correct_templates(self):
        if self.snippet_to_doc.code_snippet_type == "class_definition":
            self._set_class_documentation()
        elif (
            self.snippet_to_doc.parent_type == "class_definition"
            and self.snippet_to_doc.code_snippet_type == "function_definition"
        ):
            self._set_method_documentation()
        elif (
            self.snippet_to_doc.parent_type == "root node"
            and self.snippet_to_doc == "function_defintion"
        ):
            self._set_function_documentation()

    def _set_function_documentation(self) -> None:
        self._set_function_system_message_template()
        self._set_function_doc_message_template()
        self._set_function_current_chat_message()

    def _set_function_system_message_template(self) -> None:
        template = PromptTemplate(
            template="You are a senior python developer. Given the following function called '{function_identifier}', documentate the  function'{function_identifier}'. Take into consideration that the function belongs to the file '{file_name}' and this file have the following dependencies:{file_dependencies}. The documentation must be outputted in markdown format maintaining the following format: \n {function_format}",
            input_variables=[
                "function_identifier",
                "file_name",
                "file_dependencies",
                "function_format",
            ],
        )
        self.system_message = SystemMessagePromptTemplate(prompt=template)

    def _set_function_doc_message_template(self) -> None:
        template = PromptTemplate(
            template="Here is the implementaton of the function '{function_identifier}':\n'''{function_implementation}'''\n document the function taking your time analyzing the logic of the function, the arguments of the function, the return type, etc. Think about the correct understandment of the documentation. Use only the provided info.",
            input_variables=["function_identifier", "function_implementation"],
        )
        self.current_doc_message = HumanMessagePromptTemplate(prompt=template)

    def _set_function_current_chat_message(self):
        chat_template = ChatPromptTemplate.from_messages(
            [self.system_message, self.current_doc_message]
        )
        prompt_value = chat_template.format_prompt(
            function_identifier=self.snippet_to_doc.code_snippet_identifier,
            file_name=self.snippet_to_doc.file_name,
            file_dependencies=self.snippet_to_doc.dependencies,
            function_implementation=self.snippet_to_doc.implementation,
            function_format=FUNCTION_DOCUMENTATION_TEMPLATE,
        ).to_messages()
        self.current_chat_message = prompt_value

    def _set_class_documentation(self) -> None:
        self._set_class_system_message_template()
        self._set_class_doc_message_template()
        self._set_class_current_chat_message()

    def _set_class_system_message_template(self) -> None:
        template = PromptTemplate(
            template="You are a senior python developer. Given the following class called '{class_identifier}', documentate the clas '{class_identifier}'. Take into consideration that the class belongs to the file '{file_name}' and this file have the following dependencies:{file_dependencies}. The documentation must be outputted in markdown format maintaining the following format: \n {class_format}",
            input_variables=[
                "class_identifier",
                "file_name",
                "file_dependencies",
                "class_format",
            ],
        )
        self.system_message = SystemMessagePromptTemplate(prompt=template)

    def _set_class_doc_message_template(self) -> None:
        template = PromptTemplate(
            template="Here is the implementaton of the class '{class_identifier}':\n'''{class_implementation}'''\n document the class taking your time infering the use of the attributes, or in case of no attributes how the class would work based on the class name. The methods of the class are not relevant now because they will be document later, think about the correct understandment of the documentation. Use only the provided info.",
            input_variables=["class_identifier", "class_implementation"],
        )
        self.current_doc_message = HumanMessagePromptTemplate(prompt=template)

    def _set_class_current_chat_message(self):
        chat_template = ChatPromptTemplate.from_messages(
            [self.system_message, self.current_doc_message]
        )
        prompt_value = chat_template.format_prompt(
            class_identifier=self.snippet_to_doc.code_snippet_identifier,
            file_name=self.snippet_to_doc.file_name,
            file_dependencies=self.snippet_to_doc.dependencies,
            class_implementation=self.snippet_to_doc.implementation,
            class_format=CLASS_DOCUMENTATION_TEMPLATE,
        ).to_messages()
        self.current_chat_message = prompt_value

    def _set_method_documentation(self) -> None:
        self._set_method_system_message_template()
        self._set_method_doc_message_template()
        self._set_method_current_chat_message()

    def _set_method_system_message_template(self) -> None:
        template = PromptTemplate(
            template="You are a senior python developer. Given the following method called '{method_identifier}', that belongs to the class '{class_identifier}', documentate the method '{method_identifier}'. Take into consideration that the method an it's class belongs to the file '{file_name}' and this file have the following dependencies:{file_dependencies}. The documentation must be outputted in markdown format maintaining the following format: \n {method_format}",
            input_variables=[
                "method_identifier",
                "class_identifier",
                "file_name",
                "file_dependencies",
                "method_format",
            ],
        )
        self.system_message = SystemMessagePromptTemplate(prompt=template)

    def _set_method_doc_message_template(self) -> None:
        template = PromptTemplate(
            template="Here is the implementation of the method '{method_identifier}':\n'''{method_implementation}'''\n document the method taking your time to extract the most important things for a correct understandment of the documentation. Use only the provided info.",
            input_variables=["method_identifier", "method_implementation"],
        )
        self.current_doc_message = HumanMessagePromptTemplate(prompt=template)

    def _set_method_current_chat_message(self):
        chat_template = ChatPromptTemplate.from_messages(
            [self.system_message, self.current_doc_message]
        )
        prompt_value = chat_template.format_prompt(
            method_identifier=self.snippet_to_doc.code_snippet_identifier,
            class_identifier=self.snippet_to_doc.parent_identifier,
            file_name=self.snippet_to_doc.file_name,
            file_dependencies=self.snippet_to_doc.dependencies,
            method_implementation=self.snippet_to_doc.implementation,
            method_format=METHOD_DOCUMENTATION_TEMPLATE,
        ).to_messages()
        self.current_chat_message = prompt_value

DocumentorChat()


class SingletonMeta(type):
    _instances = {}

    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            cls._instances[cls] = super().__call__(*args, **kwargs)
            cls.instance = cls._instances[cls]
        return cls._instances[cls]



from git_tools.git_file_paths import GitFilePaths
from git_tools.git_manager import GitManager
from repository_scrapping.file_scrapper import FileScrapper
from snippet_management.snippet_storage import SnippetStorage
from metaclasses.singleton_meta import SingletonMeta
from typing import ClassVar


class FirstRunManager(metaclass=SingletonMeta):
    instance: ClassVar = None
    _snippets_to_doc: SnippetStorage
    _first_run_scrapper: FileScrapper

    def __init__(self):
        self._snippets_to_doc = SnippetStorage()
        self._first_run_scrapper = FileScrapper()

    def _start_first_run(self):
        GitManager.select_front_commit()
        self._first_run_scrapper.scrape_specified(GitFilePaths.get_all_valid_paths())
        all_snippets_to_doc_dict = self._first_run_scrapper.storage_dict
        self._snippets_to_doc.update_storage(all_snippets_to_doc_dict)

    @staticmethod
    def start_first_run():
        FirstRunManager.instance._start_first_run()

    @staticmethod
    def get_snippets_to_doc() -> SnippetStorage:
        return FirstRunManager.instance._snippets_to_doc


FirstRunManager()



from snippet_management.node_info import NodeInfo
from tree_sitter import Node
from typing import List, Set


def _descendants_with_type(node: Node, node_type: str) -> List[Node]:
    nodes = list()
    if node.type == node_type:
        nodes.append(node)
    for child in node.children:
        nodes.extend(_descendants_with_type(child, node_type))
    return nodes


def global_node_types(root: Node, node_types: Set[str]) -> List[Node]:
    return [child for child in root.children if child.type in node_types]


def get_specified_nodes(root: Node, node_types: Set[str]) -> List[Node]:
    nodes = list()
    for node_type in node_types:
        nodes.extend(_descendants_with_type(root, node_type))
    return nodes


def get_identifier(node: Node, file_str: str) -> str:
    for child in node.children:
        if child.type == "identifier":
            return file_str[child.start_byte : child.end_byte]
    return None


def get_dependencies(root: Node) -> List[Node]:
    dependencies_types = {"import_from_statement", "import_from_statement"}
    return global_node_types(root, dependencies_types)


def get_implementation(node: Node, file_str: str) -> str:
    implementation = ""
    if node.type == "class_definition":
        implementation = _get_class_implementation(node, file_str)
    else:
        implementation = file_str[node.start_byte : node.end_byte]
    return implementation


def _get_class_implementation(node: Node, file_str: str) -> str:
    valid_types = {"typed_parameter", "expression_statement", "assignment"}
    class_implementation = [
        get_implementation(block_child, file_str)
        for child in node.children
        if child.type == "block"
        for block_child in child.children
        if block_child.type in valid_types
    ]
    implementation = "\n".join(class_implementation)
    return implementation if implementation else "NO ARGUMENTS SPECIFIED IN THIS CLASS"


def _recursive_get_nodes(
    parent_node_info: NodeInfo, types: Set[str], file_str: str
) -> List[NodeInfo]:
    nodes_list: List[NodeInfo] = list()
    for child in parent_node_info.children:
        child_node_info = NodeInfo(child, parent_node_info, file_str)
        child_nodes = _recursive_get_nodes(child_node_info, types, file_str)
        nodes_list.extend(child_nodes)
        if child.type in types:
            nodes_list.append(child_node_info)
    return nodes_list


def get_nodes(root: Node, types: Set[str], file_str: str) -> List[NodeInfo]:
    root_node_info: NodeInfo = NodeInfo(root)
    return _recursive_get_nodes(root_node_info, types, file_str)



from tree_sitter import Language
from enum import Enum
from typing import Tuple, Set

Language.build_library(
    # Store the library in the `build` directory
    "build/my-languages.so",
    # Include one or more languages
    ["tree-sitter-python"],
)


class FileExtension(Enum):
    py: Language = Language("build/my-languages.so", "python")


class LanguageNodes(Enum):
    """
    (wholecode, only global scope)
    """

    py: Tuple[Set[str], Set[str]] = (
        {"class_definition", "function_definition"},
        { "function_definition"},
    )



CLASS_DOCUMENTATION_TEMPLATE = """
## `ClassName`

#### Description:
This class represents [a specific entity or concept] and provides functionalities for [specific purposes].

#### Attributes:
- `attribute1` (`Type`): Description of the purpose and use of attribute1. Default: `default_value` (if applicable).
- `attribute2` (`Type`): Description of the purpose and use of attribute2. Default: `default_value` (if applicable).
- ...
- it could have no Attributes at all.

#### Usage Cases:

```python
# Sample code to demonstrate a common use case of instantiating and using the class
instance = ClassName(arg1, arg2)
# The rest of the example
```

#### How it Works:

Brief explanation or description of the internal workings of the class or the logic behind its main functionalities.

#### Dependencies:
- `DependencyName`: Brief description of the dependency and its role.
- Another dependency (if applicable): Description...
[In case of no dependencies display] This class does not have any dependencies.
"""
#_______________________________________________________________________________
#_______________________________________________________________________________
FUNCTION_DOCUMENTATION_TEMPLATE = """
## `function_name(argument1: Type, argument2: Type) -> ReturnType`

#### Description:
This function performs [specific action] and is typically used for [purpose].

#### Arguments:
- `argument1` (`Type`): Description of the purpose and use of argument1.
- `argument2` (`Type`): Description of the purpose and use of argument2.

#### Returns:
- `ReturnType`: Description of what the return value represents.

#### Exceptions:
- `ExceptionName`: Circumstances under which this exception is thrown.

#### Usage Cases:

```python
# Sample code to demonstrate a common use case
result_case1 = function_name(value1, value2)
print(result_case1)

# Another use case (if applicable)
result_case2 = function_name(valueA, valueB)
print(result_case2)
```

#### How it Works:

Explanation or description of the internal workings of the function or the logic behind it.

#### Dependencies:
- `DependencyName`: Brief description of the dependency and its role.
- Another dependency (if applicable): Description...
[In case of no dependencies display] This function does not have any dependencies.
"""
#_______________________________________________________________________________
#_______________________________________________________________________________
METHOD_DOCUMENTATION_TEMPLATE = """
## `method_name(parameter1: Type, parameter2: Type) -> ReturnType`

#### Description:
This method performs [specific action] and is used for [purpose]. This method belongs to the class [class name].

#### Parameters:
- `parameter1` (`Type`): Description of the purpose and use of parameter1.
- `parameterN` (`Type`): Description of the purpose and use of parameterN.

#### Returns:
- `ReturnType`: Description of what the return value represents.

#### Exceptions:
- `ExceptionName`: Circumstances under which this exception is thrown.

#### Usage Cases:

```python
# Sample code to demonstrate a common use case
result_case1 = method_name(value1, value2)
print(result_case1)

# Another use case (if applicable)
result_case2 = method_name(valueA, valueB)
print(result_case2)
```

#### How it Works:

Explanation or description of the internal workings of the method or the logic behind it.

#### Dependencies:
- `DependencyName`: Brief description of the dependency and its role.
- Another dependency (if applicable): Description...
[In case of no dependencies display] This method does not have any dependencies.
"""


from git_tools.git_file_checker import GitFileChecker
from git_tools.git_manager import GitManager
from snippet_management.snippet_storage import SnippetStorage
from snippet_management.code_snippet import CodeSnippet
from repository_scrapping.file_scrapper import FileScrapper
from metaclasses.singleton_meta import SingletonMeta
from typing import Dict, ClassVar


class ModifiedFilesManager(metaclass=SingletonMeta):
    instance: ClassVar = None
    _file_scrapper_tail: FileScrapper
    _file_scrapper_front: FileScrapper
    _snippets_to_doc: SnippetStorage
    _snippets_to_delete: SnippetStorage
    _old_deleted_snippets_dict: Dict[int, CodeSnippet]
    _new_added_snippets_dict: Dict[int, CodeSnippet]
    _modified_snippets_dict: Dict[int, CodeSnippet]

    def __init__(self):
        self._file_scrapper_front = FileScrapper()
        self._file_scrapper_tail = FileScrapper()
        self._snippets_to_doc = SnippetStorage()
        self._snippets_to_delete = SnippetStorage()
        self._old_deleted_snippets_dict = {}
        self._new_added_snippets_dict = {}
        self._modified_snippets_dict = {}

    @property
    def snippets_to_delete(self):
        return self._snippets_to_delete

    @property
    def snippets_to_doc(self):
        return self._snippets_to_doc

    def _check_modified_files(self):
        if GitFileChecker.modified:
            self._start_file_scrappers()
            self._update_all_snippets_dict()
            self._update_snippets_to_doc()
            self._update_snippets_to_delete()

    def _update_snippets_to_doc(self):
        self.snippets_to_doc.update_storage(self._new_added_snippets_dict)
        self.snippets_to_doc.update_storage(self._modified_snippets_dict)

    def _update_snippets_to_delete(self):
        self.snippets_to_delete.update_storage(self._old_deleted_snippets_dict)

    def _start_file_scrapper_front(self):
        GitManager.select_front_commit()
        self._file_scrapper_front.scrape_specified(GitFileChecker.modified)

    def _start_file_scrapper_tail(self):
        GitManager.select_tail_commit()
        self._file_scrapper_tail.scrape_specified(GitFileChecker.modified)

    def _start_file_scrappers(self):
        self._start_file_scrapper_front()
        self._start_file_scrapper_tail()

    def _update_all_snippets_dict(self):
        self._update_new_added_snippets_dict()
        self._update_old_deleted_snippets_dict()
        self._update_modified_snippets_dict()

    def _update_old_deleted_snippets_dict(self):
        front_snippets_keys = set(self._file_scrapper_front.storage_dict.keys())
        tail_snippets_keys = set(self._file_scrapper_tail.storage_dict.keys())
        deleted_snippets_hash_ints = tail_snippets_keys - front_snippets_keys
        deleted_snippets_dict = {
            hash_int: snippet
            for hash_int, snippet in self._file_scrapper_tail.storage_dict.items()
            if hash_int in deleted_snippets_hash_ints
        }
        self._old_deleted_snippets_dict = deleted_snippets_dict

    def _update_new_added_snippets_dict(self):
        front_snippets_keys = set(self._file_scrapper_front.storage_dict.keys())
        tail_snippets_keys = set(self._file_scrapper_tail.storage_dict.keys())
        added_snippets_hash_ints = front_snippets_keys - tail_snippets_keys
        added_snippets_dict = {
            hash_int: snippet
            for hash_int, snippet in self._file_scrapper_front.storage_dict.items()
            if hash_int in added_snippets_hash_ints
        }
        self._new_added_snippets_dict = added_snippets_dict

    def _update_modified_snippets_dict(self):
        front_snippets_keys = set(self._file_scrapper_front.storage_dict.keys())
        tail_snippets_keys = set(self._file_scrapper_tail.storage_dict.keys())
        modified_snippets_hash_ints = front_snippets_keys.intersection(
            tail_snippets_keys
        )
        modified_snippets_dict: Dict[int, CodeSnippet] = {}
        for hash_int in modified_snippets_hash_ints:
            front_snippet = self._file_scrapper_front.storage_dict.get(hash_int)
            tail_snippet = self._file_scrapper_tail.storage_dict.get(hash_int)
            if front_snippet.implementation != tail_snippet.implementation:
                modified_snippets_dict[hash_int] = front_snippet
        self._modified_snippets_dict = modified_snippets_dict

    @staticmethod
    def get_snippets_to_doc() -> SnippetStorage:
        return ModifiedFilesManager.instance.snippets_to_doc

    @staticmethod
    def get_snippets_to_delete() -> SnippetStorage:
        return ModifiedFilesManager.instance.snippets_to_delete

    @staticmethod
    def check_modified_files():
        return ModifiedFilesManager.instance._check_modified_files()


ModifiedFilesManager()



from setuptools import setup, find_packages

with open("requirements.txt", "r") as fh:
    requirements = fh.readlines()

setup(
    name="documentor_piedpiper",
    version="0.8",
    packages=find_packages(where="src"),
    package_dir={"": "src"},
    install_requires=[
        "tree-sitter",
        "pygit2",
        "langchain",
        "openai",
        "wheel"
    ],
)



from metaclasses.singleton_meta import SingletonMeta
from pathlib import Path
from pygit2 import Repository, Commit, Tree
from typing import ClassVar, List


class GitManager(metaclass=SingletonMeta):
    instance: ClassVar = None
    project_repo: Repository
    head_commit: Commit
    front_commit: Commit  # default = HEAD
    tail_commit: Commit  # default = HEAD^
    selected_commit: Commit  # default = HEAD^
    front_commit_tree: Tree
    tail_commit_tree: Tree
    selected_commit_tree: Tree
    project_file_paths: List[Path]

    def __init__(self, repo_path: Path = Path.cwd()):
        self.project_repo = Repository(repo_path)
        self.head_commit = self.project_repo.head.peel(Commit)
        self.front_commit = self.head_commit
        self.front_commit_tree = self.front_commit.tree
        if self.head_commit.parents[0]:
            self.tail_commit = self.head_commit.parents[0]
        else:
            raise Exception(
                "GitManager_Error: get_head_parent_commit(), No head parent commit."
            )
        self.tail_commit_tree = self.tail_commit.tree
        self.selected_commit = self.tail_commit
        self.selected_commit_tree = self.tail_commit_tree

    def _update_front_commit(self, front_commit_hash: str):
        new_front_commit = self.project_repo[front_commit_hash]
        if self.project_repo.descendant_of(self.tail_commit.id, new_front_commit.id):
            self.front_commit = new_front_commit
            self.front_commit_tree = self.front_commit.tree
        else:
            raise Exception(
                "GitManager_Error: The front commit can't be previous to the tail commit"
            )

    def _update_tail_commit(self, tail_commit_hash: str):
        new_tail_commit = self.project_repo[tail_commit_hash]
        if self.project_repo.descendant_of(new_tail_commit.id, self.front_commit.id):
            self.tail_commit = new_tail_commit
            self.tail_commit_tree = self.tail_commit.tree
        else:
            raise Exception(
                "GitManager_Error: The tail commit can't be next to the tail commit"
            )

    def _select_front_commit(self):
        self.selected_commit = self.front_commit
        self.selected_commit_tree = self.front_commit_tree

    def _select_tail_commit(self):
        self.selected_commit = self.tail_commit
        self.selected_commit_tree = self.tail_commit_tree

    def _stage_file(self, file_path: Path):
        repo: Repository = self.project_repo
        repo.index.add(file_path)
        repo.index.write()

    def _stage_file_list(self, files_paths: List[Path]):
        for file_path in files_paths:
            self._stage_file(file_path)

    def _commit(self, commit_msg: str = None):
        if not commit_msg:
            commit_msg = f"DOCUMENTED COMMIT:{self.head_commit.short_id}"
        repo: Repository = self.project_repo
        author = repo.default_signature
        tree = repo.index.write_tree()
        repo.create_commit("HEAD", author, author, commit_msg, tree, [repo.head.target])

    @staticmethod
    def stage_file(file_path: Path):
        GitManager.instance._stage_file(file_path)

    @staticmethod
    def stage_file_list(files_paths: List[Path]):
        GitManager.instance._stage_file_list(files_paths)

    @staticmethod
    def commit(commit_msg: str = None):
        GitManager.instance._commit(commit_msg)

    @staticmethod
    def commit_doc_changes():  # also should stage docs directory
        doc_log_path = Path("./doc.log")
        GitManager.stage_file(doc_log_path)
        GitManager.commit()

    @staticmethod
    def update_front_commit(front_commit_hash: str):
        GitManager.instance._update_front_commit(front_commit_hash)

    @staticmethod
    def update_tail_commit(tail_commit_hash: str):
        GitManager.instance._update_tail_commit(tail_commit_hash)

    @staticmethod
    def select_front_commit():
        GitManager.instance._select_front_commit()

    @staticmethod
    def select_tail_commit():
        GitManager.instance._select_tail_commit()

    @staticmethod
    def project_repo() -> Repository:
        return GitManager.instance.project_repo

    @staticmethod
    def head_commit() -> Commit:
        return GitManager.instance.head_commit

    @staticmethod
    def front_commit() -> Commit:
        return GitManager.instance.front_commit

    @staticmethod
    def front_commit_tree() -> Tree:
        return GitManager.instance.front_commit_tree

    @staticmethod
    def tail_commit() -> Commit:
        return GitManager.instance.tail_commit

    @staticmethod
    def tail_commit_tree() -> Tree:
        return GitManager.instance.tail_commit_tree

    @staticmethod
    def selected_commit() -> Commit:
        return GitManager.instance.selected_commit

    @staticmethod
    def selected_commit_tree() -> Tree:
        return GitManager.instance.selected_commit_tree

    @staticmethod
    def get_head_parent_commit() -> Commit:
        if GitManager.head_commit().parents[0]:
            return GitManager.head_commit().parents[0]
        else:
            raise Exception(
                "GitManager_Error: get_head_parent_commit(), No head parent commit."
            )


GitManager()
